@@ -12,10 +12,6 @@ import subprocess
 # documentation root, use os.path.abspath to make it absolute, like shown here.
 #sys.path.insert(0, os.path.abspath('.'))
 
-# Add the 'extensions' directory to sys.path, to enable finding Sphinx
-# extensions within.
-sys.path.append(os.path.join(os.path.abspath("."), 'extensions'))
-
 # -- General configuration ------------------------------------------------
 
 # Add any Sphinx extension module names here, as strings. They can be
@@ -48,7 +44,7 @@ master_doc = 'index'
 # General information about the project.
 project = u'RAUC'
 author = u'Jan Luebbe, Enrico Joerns, the RAUC contributors'
-copyright = u'2016-2024, ' + author
+copyright = u'2016-2025, ' + author
 
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the
@@ -68,11 +64,6 @@ exclude_patterns = []
 # The name of the Pygments (syntax highlighting) style to use.
 pygments_style = 'sphinx'
 
-# Additional lexer for Pygments (syntax highlighting)
-from lexer.DtsLexer import DtsLexer
-from sphinx.highlighting import lexers
-lexers['DTS'] = DtsLexer()
-
 # -- Options for HTML output ----------------------------------------------
 
 # The theme to use for HTML and HTML Help pages.  See the documentation for
