@@ -14,6 +14,7 @@
 #include <unistd.h>
 
 #include "bundle.h"
+#include "config_file.h"
 #include "context.h"
 #include "crypt.h"
 #include "manifest.h"
@@ -98,15 +99,13 @@
 #define ZFS_SUPER_MAGIC 0x2fc12fc1
 #endif
 
-#define MAX_BUNDLE_SIGNATURE_SIZE 0x10000
-
 GQuark
 r_bundle_error_quark(void)
 {
 	return g_quark_from_static_string("r-bundle-error-quark");
 }
 
-static gboolean mksquashfs(const gchar *bundlename, const gchar *contentdir, const gchar *fakeroot, GError **error)
+static gboolean mksquashfs(const gchar *bundlename, const gchar *contentdir, gboolean keep_metadata, const gchar *fakeroot, GError **error)
 {
 	GError *ierror = NULL;
 	gboolean res = FALSE;
@@ -130,7 +129,7 @@ static gboolean mksquashfs(const gchar *bundlename, const gchar *contentdir, con
 	g_ptr_array_add(args, g_strdup(bundlename));
 	g_ptr_array_add(args, g_strdup("-noappend"));
 	g_ptr_array_add(args, g_strdup("-no-progress"));
-	if (!fakeroot) {
+	if (!keep_metadata) {
 		g_ptr_array_add(args, g_strdup("-all-root"));
 		g_ptr_array_add(args, g_strdup("-no-xattrs"));
 	}
@@ -507,6 +506,58 @@ static gchar *convert_tar_extract(RaucImage *image, const gchar *dir, const gcha
 	return g_steal_pointer(&converted);
 }
 
+#if ENABLE_COMPOSEFS == 1
+static gchar *convert_composefs(RaucImage *image, const gchar *dir, const gchar *tar_extracted_path, const gchar *fakeroot, GError **error)
+{
+	GError *ierror = NULL;
+	g_autoptr(GPtrArray) args = g_ptr_array_new_full(10, g_free);
+
+	g_return_val_if_fail(image, NULL);
+	g_return_val_if_fail(dir, NULL);
+	g_return_val_if_fail(tar_extracted_path, NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
+
+	g_autofree gchar *converted = g_strdup_printf("%s.cfs", image->filename);
+	g_autofree gchar *converted_path = g_build_filename(dir, converted, NULL);
+	g_autofree gchar *converted_image_path = g_build_filename(converted_path, "image.cfs", NULL);
+	g_autofree gchar *store_path = g_build_filename(dir, ".rauc-cfs-store", NULL);
+
+	/* Create a directory in the bundle so we can keep additional metadata, like
+	 * a signature over the image digest.
+	 */
+	if (g_mkdir(converted_path, S_IRWXU)) {
+		int err = errno;
+		g_set_error(error, G_FILE_ERROR, g_file_error_from_errno(err),
+				"failed to create directory '%s': %s", converted_path, g_strerror(err));
+		return NULL;
+	}
+
+	if (g_mkdir_with_parents(store_path, S_IRWXU)) {
+		int err = errno;
+		g_set_error(error, G_FILE_ERROR, g_file_error_from_errno(err),
+				"failed to create directory '%s': %s", store_path, g_strerror(err));
+		return NULL;
+	}
+
+	r_fakeroot_add_args(args, fakeroot);
+
+	g_ptr_array_add(args, g_strdup("mkcomposefs"));
+	g_ptr_array_add(args, g_strdup_printf("--digest-store=%s", store_path));
+	/* TODO mkcomposefs fails to read xattrs from broken symlinks under pseudo. */
+	g_ptr_array_add(args, g_strdup("--skip-xattrs"));
+	g_ptr_array_add(args, g_strdup(tar_extracted_path));
+	g_ptr_array_add(args, g_strdup(converted_image_path));
+	g_ptr_array_add(args, NULL);
+
+	if (!r_subprocess_runv(args, G_SUBPROCESS_FLAGS_NONE, &ierror)) {
+		g_propagate_prefixed_error(error, ierror, "failed to run mkcomposefs: ");
+		return NULL;
+	}
+
+	return g_steal_pointer(&converted);
+}
+#endif
+
 static gboolean convert_images(RaucManifest *manifest, const gchar *dir, const gchar *fakeroot, GError **error)
 {
 	GError *ierror = NULL;
@@ -530,17 +581,39 @@ static gboolean convert_images(RaucManifest *manifest, const gchar *dir, const g
 		if (!image->convert)
 			continue;
 
+		g_autofree gchar *tar_extracted = NULL;
+		g_autofree gchar *tar_extracted_path = NULL;
+		/* extract tar early if we need it for other outputs */
+		if (g_strv_contains((const gchar * const *)image->convert, "tar-extract") ||
+		    g_strv_contains((const gchar * const *)image->convert, "composefs")) {
+			g_debug("extracting tar artifact image '%s'", image->filename);
+			tar_extracted = convert_tar_extract(image, dir, fakeroot, &ierror);
+			if (!tar_extracted) {
+				g_propagate_error(error, ierror);
+				return FALSE;
+			}
+			tar_extracted_path = g_build_filename(dir, tar_extracted, NULL);
+		}
+
 		gboolean keep = FALSE;
 		g_autoptr(GPtrArray) converted = g_ptr_array_new_with_free_func(g_free);
 		for (gchar **method = image->convert; *method != NULL; method++) {
 			gchar *converted_filename = NULL;
 			if (g_str_equal(*method, "tar-extract")) {
-				g_debug("extracting tar artifact image '%s'", image->filename);
-				converted_filename = convert_tar_extract(image, dir, fakeroot, &ierror);
+				converted_filename = g_strdup(tar_extracted);
+			} else if (g_str_equal(*method, "composefs")) {
+#if ENABLE_COMPOSEFS == 1
+				g_debug("converting '%s' to composefs image", tar_extracted_path);
+				converted_filename = convert_composefs(image, dir, tar_extracted_path, fakeroot, &ierror);
 				if (!converted_filename) {
 					g_propagate_error(error, ierror);
 					return FALSE;
 				}
+#else
+				g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_UNSUPPORTED,
+						"Convert method 'composefs' not enabled, recompile with -Dcomposefs=enabled");
+				return FALSE;
+#endif
 			} else if (g_str_equal(*method, "keep")) {
 				g_debug("keeping input artifact image '%s'", image->filename);
 				keep = TRUE;
@@ -558,6 +631,13 @@ static gboolean convert_images(RaucManifest *manifest, const gchar *dir, const g
 			g_ptr_array_add(converted, converted_filename);
 		}
 
+		if (tar_extracted_path && !g_strv_contains((const gchar * const *)image->convert, "tar-extract")) {
+			if (!rm_tree(tar_extracted_path, &ierror)) {
+				g_propagate_prefixed_error(error, ierror, "Failed to remove files extacted from tar: ");
+				return FALSE;
+			}
+		}
+
 		if (!keep) {
 			g_autofree gchar *file_path = g_build_filename(dir, image->filename, NULL);
 			g_debug("removing input artifact image '%s' after conversion", image->filename);
@@ -777,6 +857,11 @@ static gboolean append_signature_to_bundle(const gchar *bundlename, GBytes *sig,
 	}
 
 	offset = g_seekable_tell(G_SEEKABLE(bundlestream)) - offset;
+	if (offset > DEFAULT_MAX_BUNDLE_SIGNATURE_SIZE) {
+		g_autofree gchar* formatted_sigsize = g_format_size_full(offset, G_FORMAT_SIZE_IEC_UNITS);
+		g_autofree gchar* formatted_defaultsize = g_format_size_full(DEFAULT_MAX_BUNDLE_SIGNATURE_SIZE, G_FORMAT_SIZE_IEC_UNITS);
+		g_message("Resulting signature size of %s exceeds default of %s. Make sure to set 'max-bundle-signature-size' in your target's system.conf to a sufficiently high value.", formatted_sigsize, formatted_defaultsize);
+	}
 	if (!output_stream_write_uint64_all(bundleoutstream, offset, NULL, &ierror)) {
 		g_propagate_prefixed_error(
 				error,
@@ -1131,22 +1216,42 @@ static gchar *prepare_workdir(const gchar *contentdir, GError **error)
 	return g_steal_pointer(&workdir);
 }
 
-static gboolean needs_fakeroot(const RaucManifest *manifest)
+static gboolean needs_fakeroot(const RaucManifest *manifest, gboolean *mksquashfs_metadata)
 {
-	/* If pseudo is active, we don't need to use fakeroot. */
-	if (check_pseudo_active())
-		return FALSE;
+	g_return_val_if_fail(manifest != NULL, FALSE);
+	g_return_val_if_fail(mksquashfs_metadata != NULL, FALSE);
 
+	*mksquashfs_metadata = FALSE;
+
+	gboolean may_need_fakeroot = FALSE;
 	for (GList *elem = manifest->images; elem != NULL; elem = elem->next) {
 		RaucImage *image = elem->data;
 
 		if (image->convert) {
-			if (g_strv_contains((const gchar * const *)image->convert, "tar-extract"))
-				return TRUE;
+			if (g_strv_contains((const gchar * const *)image->convert, "tar-extract")) {
+				/* This produces files with metadata in the bundle itself. */
+				*mksquashfs_metadata = TRUE;
+				may_need_fakeroot = TRUE;
+			}
+			if (g_strv_contains((const gchar * const *)image->convert, "composefs")) {
+				/* The metadata is stored in the cfs image. */
+				may_need_fakeroot = TRUE;
+			}
 		}
 	}
 
-	return FALSE;
+	if (!may_need_fakeroot)
+		return FALSE;
+
+	/* If pseudo is active, we don't need to use fakeroot. */
+	if (check_pseudo_active())
+		return FALSE;
+
+	/* When running as root, we don't need to use fakeroot. */
+	if (geteuid() == 0)
+		return FALSE;
+
+	return TRUE;
 }
 
 gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError **error)
@@ -1156,6 +1261,7 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 	g_autoptr(RaucManifest) manifest = NULL;
 	g_autofree gchar *workdir = NULL;
 	g_autofree gchar *fakeroot = NULL;
+	gboolean mksquashfs_metadata = FALSE;
 	gboolean res = FALSE;
 
 	g_return_val_if_fail(bundlename != NULL, FALSE);
@@ -1206,7 +1312,7 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 		goto out;
 	}
 
-	if (needs_fakeroot(manifest)) {
+	if (needs_fakeroot(manifest, &mksquashfs_metadata)) {
 		fakeroot = r_fakeroot_init(&ierror);
 		if (!fakeroot) {
 			g_propagate_error(error, ierror);
@@ -1232,7 +1338,7 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 		goto out;
 	}
 
-	res = mksquashfs(bundlename, workdir, fakeroot, &ierror);
+	res = mksquashfs(bundlename, workdir, mksquashfs_metadata, fakeroot, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -1513,7 +1619,7 @@ static gboolean convert_to_casync_bundle(RaucBundle *bundle, const gchar *outbun
 		goto out;
 	}
 
-	res = mksquashfs(outbundle, contentdir, NULL, &ierror);
+	res = mksquashfs(outbundle, contentdir, FALSE, NULL, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
 		goto out;
@@ -2030,10 +2136,11 @@ static gboolean open_local_bundle(RaucBundle *bundle, GError **error)
 		res = FALSE;
 		goto out;
 	}
-	/* sanity check: signature should be smaller than 64KiB */
-	if (sigsize > MAX_BUNDLE_SIGNATURE_SIZE) {
+	/* sanity check: signature should not exceed the configured maximum */
+	if (sigsize > r_context()->config->max_bundle_signature_size) {
 		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE,
-				"Signature size (%"G_GUINT64_FORMAT ") exceeds 64KiB", sigsize);
+				"Signature size (%"G_GUINT64_FORMAT ") exceeds max-bundle-signature-size (%"G_GUINT64_FORMAT ")",
+				sigsize, r_context()->config->max_bundle_signature_size);
 		res = FALSE;
 		goto out;
 	}
@@ -2120,10 +2227,11 @@ static gboolean open_remote_bundle(RaucBundle *bundle, GError **error)
 		res = FALSE;
 		goto out;
 	}
-	/* sanity check: signature should be smaller than 64KiB */
-	if (sigsize > MAX_BUNDLE_SIGNATURE_SIZE) {
+	/* sanity check: signature should not exceed the configured maximum */
+	if (sigsize > r_context()->config->max_bundle_signature_size) {
 		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE,
-				"Signature size (%"G_GUINT64_FORMAT ") exceeds 64KiB", sigsize);
+				"Signature size (%"G_GUINT64_FORMAT ") exceeds max-bundle-signature-size (%"G_GUINT64_FORMAT ")",
+				sigsize, r_context()->config->max_bundle_signature_size);
 		res = FALSE;
 		goto out;
 	}
