@@ -3,6 +3,7 @@
 #include <glib/gstdio.h>
 #include <string.h>
 
+#include "bootchooser.h"
 #include "config_file.h"
 #include "context.h"
 #include "event_log.h"
@@ -69,8 +70,10 @@ static gchar* get_cmdline_bootname(void)
 	else if (!g_file_get_contents("/proc/cmdline", &contents, NULL, NULL))
 		return NULL;
 
-	if (strstr(contents, "rauc.external") != NULL)
+	if (strstr(contents, "rauc.external") != NULL) {
+		g_message("Detected explicit external boot, ignoring missing active slot");
 		return g_strdup("_external_");
+	}
 
 	bootname = regex_match("rauc\\.slot=(\\S+)", contents);
 	if (bootname)
@@ -87,6 +90,11 @@ static gchar* get_cmdline_bootname(void)
 	}
 
 	bootname = regex_match("root=(\\S+)", contents);
+	if (g_strcmp0(bootname, "/dev/nfs") == 0) {
+		g_message("Detected nfs boot, ignoring missing active slot");
+		g_free(bootname);
+		return g_strdup("_external_");
+	}
 	if (!bootname)
 		bootname = regex_match("systemd\\.verity_root_data=(\\S+)", contents);
 
@@ -140,57 +148,6 @@ static gchar* get_cmdline_bootname(void)
 	return bootname;
 }
 
-static gchar* get_custom_bootname(void)
-{
-	g_autoptr(GSubprocessLauncher) launcher = NULL;
-	g_autoptr(GSubprocess) handle = NULL;
-	g_autoptr(GDataInputStream) datainstream = NULL;
-	g_autoptr(GPtrArray) args_array = NULL;
-	g_autofree gchar *outline = NULL;
-	GError *ierror = NULL;
-	GInputStream *instream;
-	int res;
-
-	args_array = g_ptr_array_new();
-	g_ptr_array_add(args_array, context->config->custom_bootloader_backend);
-	g_ptr_array_add(args_array, (gchar *)("get-current"));
-	g_ptr_array_add(args_array, NULL);
-
-	launcher = g_subprocess_launcher_new(G_SUBPROCESS_FLAGS_STDOUT_PIPE);
-	handle = r_subprocess_launcher_spawnv(launcher, args_array, NULL);
-
-	if (handle == NULL) {
-		g_message("Failed to run custom backend '%s'",
-				context->config->custom_bootloader_backend);
-		return NULL;
-	}
-
-	instream = g_subprocess_get_stdout_pipe(handle);
-	datainstream = g_data_input_stream_new(instream);
-
-	outline = g_data_input_stream_read_line(datainstream, NULL, NULL, &ierror);
-
-	if (ierror) {
-		g_message("Failed to read custom backend output");
-		return NULL;
-	}
-
-	res = g_subprocess_wait_check(handle, NULL, NULL);
-	if (!res) {
-		g_message("Failed to get custom backend output");
-		return NULL;
-	}
-
-	if (!outline) {
-		g_message("Failed to get custom backend bootname: no output");
-		return NULL;
-	}
-
-	g_debug("Resolved custom backend bootname to %s", outline);
-
-	return g_steal_pointer(&outline);
-}
-
 /**
  * Launches a handler and obtains variables from output by looking for
  * 'RAUC_<SOMETHING>=value' lines to put them into a key/value store (GHashTable).
@@ -328,12 +285,14 @@ static GHashTable *get_system_info_from_handler(GError **error)
 
 	g_hash_table_iter_init(&iter, vars);
 	while (g_hash_table_iter_next(&iter, (gpointer*) &key, (gpointer*) &value)) {
-		/* legacy handling */
+		/* handle special-purpose variables */
 		if (g_strcmp0(key, "RAUC_SYSTEM_SERIAL") == 0) {
 			r_replace_strdup(&context->system_serial, value);
 		} else if (g_strcmp0(key, "RAUC_SYSTEM_VARIANT") == 0) {
 			/* set variant (overrides possible previous value) */
 			r_replace_strdup(&context->config->system_variant, value);
+		} else if (g_strcmp0(key, "RAUC_SYSTEM_VERSION") == 0) {
+			r_replace_strdup(&context->system_version, value);
 		}
 	}
 
@@ -418,17 +377,23 @@ static gboolean r_context_configure_target(GError **error)
 		context->bootslot = get_cmdline_bootname();
 	}
 
-	if (context->bootslot == NULL && (g_strcmp0(context->config->system_bootloader, "custom") == 0)) {
-		context->bootslot = get_custom_bootname();
+	if (context->bootslot == NULL) {
+		context->bootslot = r_boot_get_current_bootname(context->config, &ierror);
+		if (ierror) {
+			g_message("Failed to get bootname: %s", ierror->message);
+			g_clear_error(&ierror);
+		}
 	}
 
 	g_clear_pointer(&context->boot_id, g_free);
 	g_clear_pointer(&context->machine_id, g_free);
 
 	context->boot_id = get_boot_id();
-	g_debug("Obtained system boot ID: '%s'", context->boot_id);
+	if (context->boot_id)
+		g_debug("Obtained system boot ID: '%s'", context->boot_id);
 	context->machine_id = get_machine_id();
-	g_debug("Obtained system machine ID: '%s'", context->machine_id);
+	if (context->machine_id)
+		g_debug("Obtained system machine ID: '%s'", context->machine_id);
 
 	return TRUE;
 }
@@ -775,6 +740,10 @@ void r_context_set_step_percentage(const gchar *name, gint custom_percent)
 
 	percent_difference = custom_percent - step->last_explicit_percent;
 
+	/* skip progress update if percentage did not change */
+	if (percent_difference < 1)
+		return;
+
 	step->percent_done = step->percent_total
 	                     * (percent_difference / 100.0f);
 
@@ -885,6 +854,7 @@ void r_context_clean(void)
 		g_clear_pointer(&context->boot_id, g_free);
 		g_clear_pointer(&context->machine_id, g_free);
 		g_clear_pointer(&context->system_serial, g_free);
+		g_clear_pointer(&context->system_version, g_free);
 		g_clear_pointer(&context->system_info, g_hash_table_destroy);
 
 		g_clear_pointer(&context->handlerextra, g_free);
