@@ -52,6 +52,7 @@ gchar **recipients = NULL;
 gchar *handler_args = NULL;
 gchar *bootslot = NULL;
 gchar *installation_txn = NULL;
+gchar *require_manifest_hash = NULL;
 gboolean utf8_supported = FALSE;
 RaucBundleAccessArgs access_args = {0};
 
@@ -264,6 +265,7 @@ static gboolean install_start(int argc, char **argv)
 	args->ignore_compatible = install_ignore_compatible;
 	args->ignore_version_limit = install_ignore_version_limit;
 	args->transaction = installation_txn;
+	args->require_manifest_hash = require_manifest_hash;
 	if (access_args.tls_cert)
 		args->access_args.tls_cert = g_strdup(access_args.tls_cert);
 	if (access_args.tls_key)
@@ -285,6 +287,8 @@ static gboolean install_start(int argc, char **argv)
 		g_variant_dict_insert(&dict, "ignore-version-limit", "b", args->ignore_version_limit);
 		if (args->transaction)
 			g_variant_dict_insert(&dict, "transaction-id", "s", args->transaction);
+		if (args->require_manifest_hash)
+			g_variant_dict_insert(&dict, "require-manifest-hash", "s", args->require_manifest_hash);
 		if (args->access_args.tls_cert)
 			g_variant_dict_insert(&dict, "tls-cert", "s", args->access_args.tls_cert);
 		if (args->access_args.tls_key)
@@ -511,6 +515,7 @@ static gboolean write_slot_start(int argc, char **argv)
 	update_handler = get_update_handler(image, slot, &ierror);
 	if (update_handler == NULL) {
 		g_printerr("%s\n", ierror->message);
+		g_clear_error(&ierror);
 		r_exit_status = 1;
 		goto out;
 	}
@@ -2369,7 +2374,9 @@ static gboolean service_start(int argc, char **argv)
 
 	if (r_context()->system_status) {
 		/* Boot ID-based system reboot vs service restart detection */
-		if (g_strcmp0(r_context()->system_status->boot_id, r_context()->boot_id) == 0) {
+		if (!r_context()->boot_id) {
+			g_message("Kernel does not support reading boot_id. Skipping reboot detection.");
+		} else if (g_strcmp0(r_context()->system_status->boot_id, r_context()->boot_id) == 0) {
 			r_event_log_message(R_EVENT_LOG_TYPE_SERVICE, "Service restarted");
 		} else {
 			if (g_strcmp0(r_context()->bootslot, "_external_") == 0) {
@@ -2492,6 +2499,7 @@ static GOptionEntry entries_install[] = {
 	{"ignore-compatible", '\0', 0, G_OPTION_ARG_NONE, &install_ignore_compatible, "disable compatible check", NULL},
 	{"ignore-version-limit", '\0', 0, G_OPTION_ARG_NONE, &install_ignore_version_limit, "disable version check", NULL},
 	{"transaction-id", '\0', 0, G_OPTION_ARG_STRING, &installation_txn, "custom transaction id", "UUID"},
+	{"require-manifest-hash", '\0', 0, G_OPTION_ARG_STRING, &require_manifest_hash, "require a specific manifest hash", "HASH"},
 #if ENABLE_SERVICE == 1
 	{"progress", '\0', 0, G_OPTION_ARG_NONE, &install_progressbar, "show progress bar", NULL},
 #else
@@ -2979,7 +2987,7 @@ int main(int argc, char **argv)
 		utf8_supported = TRUE;
 
 	if (ENABLE_STREAMING && g_getenv("RAUC_NBD_SERVER")) {
-		GError *ierror = NULL;
+		g_autoptr(GError) ierror = NULL;
 		pthread_setname_np(pthread_self(), "rauc-nbd");
 		if (r_nbd_run_server(RAUC_SOCKET_FD, &ierror)) {
 			return 0;
