@@ -111,6 +111,9 @@ static gboolean r_on_handle_install_bundle(
 	if (g_variant_dict_lookup(&dict, "transaction-id", "s", &args->transaction))
 		g_variant_dict_remove(&dict, "transaction-id");
 
+	if (g_variant_dict_lookup(&dict, "require-manifest-hash", "s", &args->require_manifest_hash))
+		g_variant_dict_remove(&dict, "require-manifest-hash");
+
 	convert_dict_to_bundle_access_args(&dict, &args->access_args);
 
 	/* Check for unhandled keys */
@@ -163,7 +166,7 @@ static gboolean r_on_handle_inspect_bundle(RInstaller *interface,
 		GDBusMethodInvocation  *invocation,
 		const gchar *arg_bundle, GVariant *arg_args)
 {
-	RaucBundleAccessArgs access_args = {0};
+	g_auto(RaucBundleAccessArgs) access_args = {0};
 	g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(arg_args);
 	g_autoptr(GVariant) remaining = NULL;
 	GVariantIter iter;
