@@ -316,7 +316,8 @@ static STACK_OF(X509) *load_certs_from_file(const gchar *certfile, GError **erro
 		if (cert_x509 == NULL) {
 			err = ERR_peek_last_error();
 
-			if (ERR_GET_REASON(err) == PEM_R_NO_START_LINE) {
+			if (ERR_GET_REASON(err) == PEM_R_NO_START_LINE &&
+			    sk_X509_num(certs) != 0) {
 				/* simply reached end of file */
 				ERR_clear_error();
 				break;
@@ -1132,9 +1133,10 @@ static void debug_cms_ci(CMS_ContentInfo *cms)
 	gchar *out_str = NULL;
 	long size;
 
-	if ((domains == NULL) || (strstr(domains, G_LOG_DOMAIN "-signature") == NULL)) {
+	if (domains == NULL)
+		return;
+	if (!g_str_equal(domains, "all") && strstr(domains, G_LOG_DOMAIN "-signature") == NULL)
 		return;
-	}
 
 	out = BIO_new(BIO_s_mem());
 	CMS_ContentInfo_print_ctx(out, cms, 2, NULL);
