@@ -10,7 +10,9 @@ from random import Random
 from contextlib import contextmanager
 
 import pytest
-from pydbus import SessionBus
+from dasbus.connection import SessionMessageBus
+from dasbus.error import DBusError
+import requests
 
 from helper import run
 
@@ -154,6 +156,9 @@ def have_service():
 needs_emmc = pytest.mark.skipif("RAUC_TEST_EMMC" not in os.environ, reason="Missing eMMC")
 
 
+needs_composefs = pytest.mark.skipif(not string_in_config_h("ENABLE_COMPOSEFS 1"), reason="Missing composefs support")
+
+
 def softhsm2_load_key_pair(cert, privkey, label, id_, softhsm2_mod):
     proc = subprocess.run(
         f"openssl x509 -in {cert} -inform pem -outform der "
@@ -296,9 +301,6 @@ def pkcs11(tmp_path_factory):
 
 @pytest.fixture(scope="session")
 def dbus_session_bus(tmp_path_factory):
-    if not have_service():
-        pytest.skip("No service")
-
     addr_r_fd, addr_w_fd = os.pipe()
 
     try:
@@ -343,6 +345,7 @@ def create_system_files(env_setup, tmp_path):
     os.mkdir(tmp_path / "repos")
     os.mkdir(tmp_path / "repos/files")
     os.mkdir(tmp_path / "repos/trees")
+    os.mkdir(tmp_path / "repos/composefs")
     os.symlink(os.path.abspath("bin"), tmp_path / "bin")
     os.symlink(os.path.abspath("openssl-ca"), tmp_path / "openssl-ca")
     os.symlink(os.path.abspath("openssl-enc"), tmp_path / "openssl-enc")
@@ -359,7 +362,7 @@ def rauc_no_service(create_system_files, tmp_path):
 
 
 @pytest.fixture
-def rauc_dbus_service_with_system(tmp_path, dbus_session_bus, create_system_files, system):
+def rauc_dbus_service_with_system(tmp_path, create_system_files, system):
     system.prepare_minimal_config()
     system.write_config()
     with system.running_service("A"):
@@ -367,7 +370,7 @@ def rauc_dbus_service_with_system(tmp_path, dbus_session_bus, create_system_file
 
 
 @pytest.fixture
-def rauc_dbus_service_with_system_crypt(tmp_path, dbus_session_bus, create_system_files, system):
+def rauc_dbus_service_with_system_crypt(tmp_path, create_system_files, system):
     system.prepare_crypt_config()
     system.write_config()
     with system.running_service("A"):
@@ -375,7 +378,7 @@ def rauc_dbus_service_with_system_crypt(tmp_path, dbus_session_bus, create_syste
 
 
 @pytest.fixture
-def rauc_dbus_service_with_system_external(tmp_path, dbus_session_bus, create_system_files, system):
+def rauc_dbus_service_with_system_external(tmp_path, create_system_files, system):
     system.prepare_minimal_config()
     system.write_config()
     with system.running_service("_external_"):
@@ -383,13 +386,29 @@ def rauc_dbus_service_with_system_external(tmp_path, dbus_session_bus, create_sy
 
 
 @pytest.fixture
-def rauc_dbus_service_with_system_adaptive(tmp_path, dbus_session_bus, create_system_files, system):
+def rauc_dbus_service_with_system_adaptive(tmp_path, create_system_files, system):
     system.prepare_adaptive_config()
     system.write_config()
     with system.running_service("A"):
         yield system.proxy
 
 
+@pytest.fixture
+def rauc_dbus_service_with_system_abc(tmp_path, create_system_files, system):
+    system.prepare_abc_config()
+    system.write_config()
+    with system.running_service("A"):
+        yield system.proxy
+
+
+@pytest.fixture
+def rauc_dbus_service_with_system_composefs(tmp_path, create_system_files, system):
+    system.prepare_composefs_config()
+    system.write_config()
+    with system.running_service("A"):
+        yield system.proxy
+
+
 class Bundle:
     def __init__(self, tmp_path, name=None):
         if name is None:
@@ -534,12 +553,44 @@ class System:
             "post-install": "bin/postinstall.sh",
         }
 
+    def prepare_abc_config(self):
+        self.prepare_minimal_config()
+        # add third slot group
+        self.config["slot.rootfs.2"] = {
+            "device": "images/rootfs-2",
+            "type": "raw",
+            "bootname": "C",
+        }
+        self.config["slot.appfs.2"] = {
+            "device": "images/appfs-2",
+            "type": "raw",
+            "parent": "rootfs.2",
+        }
+        # create target devices for third slot group
+        open(self.tmp_path / "images/rootfs-2", mode="w").close()
+        open(self.tmp_path / "images/appfs-2", mode="w").close()
+        # prepare grub env for 3 slots
+        run(
+            f'grub-editenv {self.tmp_path}/grubenv.test set ORDER="A B C" A_TRY="0" B_TRY="0" C_TRY="0" A_OK="1" B_OK="1" C_OK="1"'
+        )
+
+    def prepare_composefs_config(self):
+        self.prepare_minimal_config()
+        self.config["artifacts.composefs"] = {
+            "path": "repos/composefs",
+            "type": "composefs",
+        }
+
     def write_config(self):
         with open(self.output, "w") as f:
             self.config.write(f, space_around_delimiters=False)
 
     @contextmanager
     def running_service(self, bootslot):
+        if not have_service():
+            # TODO avoid unnescesary setup by moving using a pytest mark for all service/noservice cases
+            pytest.skip("No service")
+
         assert self.service is None
         assert self.proxy is None
 
@@ -551,16 +602,17 @@ class System:
             env=env,
         )
 
-        bus = SessionBus()
+        bus = SessionMessageBus()
 
         # Wait for de.pengutronix.rauc to appear on the bus
         timeout = time.monotonic() + 5.0
         while True:
             time.sleep(0.1)
             try:
-                self.proxy = bus.get("de.pengutronix.rauc", "/")
+                self.proxy = bus.get_proxy("de.pengutronix.rauc", "/")
+                self.proxy.Operation  # try to access the service
                 break
-            except Exception:
+            except DBusError:
                 if time.monotonic() > timeout:
                     raise
 
@@ -569,13 +621,49 @@ class System:
         self.service.terminate()
         try:
             self.service.wait(timeout=10)
+            assert self.service.returncode == 0
         except subprocess.TimeoutExpired:
             self.service.kill()
             self.service.wait()
 
 
 @pytest.fixture
-def system(tmp_path):
+def system(tmp_path, dbus_session_bus):
     system = System(tmp_path)
 
     yield system
+
+
+class HTTPServer:
+    BASE = "http://127.0.0.1/backend"
+
+    def __init__(self):
+        self.url = f"{self.BASE}/get"
+
+    def setup(self, *, file_path):
+        resp = requests.post(
+            f"{self.BASE}/setup",
+            timeout=5,
+            json={
+                "file_path": file_path,
+            },
+        )
+        resp.raise_for_status()
+
+    def get(self, **kwargs):
+        kwargs.setdefault("timeout", 15)
+        return requests.get(self.url, **kwargs)
+
+    def head(self, **kwargs):
+        kwargs.setdefault("timeout", 5)
+        return requests.head(self.url, **kwargs)
+
+    def get_summary(self):
+        resp = requests.get(f"{self.BASE}/summary", timeout=15)
+        resp.raise_for_status()
+        return resp.json()
+
+
+@pytest.fixture
+def http_server():
+    return HTTPServer()
