@@ -61,6 +61,54 @@ def test_status_mark_good_non_bootslot(rauc_no_service):
     assert exitcode == 1
 
 
+@have_grub
+def test_status_mark_bad_other(rauc_dbus_service_with_system_abc):
+    out, err, exitcode = run("rauc status --output-format=json")
+    assert exitcode == 0
+    status = json.loads(out)
+
+    for slotname, property in status["slots"][0].items():
+        if property["state"] != "booted" and property["class"] == "rootfs":
+            assert property["boot_status"] == "good"
+
+    out, err, exitcode = run("rauc status mark-bad other")
+    assert exitcode == 0
+
+    out, err, exitcode = run("rauc status --output-format=json")
+    assert exitcode == 0
+    status = json.loads(out)
+
+    for slotname, property in status["slots"][0].items():
+        if property["state"] != "booted" and property["class"] == "rootfs":
+            assert property["boot_status"] == "bad"
+
+
+@have_grub
+def test_status_mark_prevent_late_fallback(tmp_path, create_system_files, system):
+    system.prepare_abc_config()
+    system.config["system"]["prevent-late-fallback"] = "true"
+    system.write_config()
+    with system.running_service("A"):
+        out, err, exitcode = run("rauc status --output-format=json")
+        assert exitcode == 0
+        status = json.loads(out)
+
+        for slotname, property in status["slots"][0].items():
+            if property["state"] != "booted" and property["class"] == "rootfs":
+                assert property["boot_status"] == "good"
+
+        out, err, exitcode = run("rauc status mark-good")
+        assert exitcode == 0
+
+        out, err, exitcode = run("rauc status --output-format=json")
+        assert exitcode == 0
+        status = json.loads(out)
+
+        for slotname, property in status["slots"][0].items():
+            if property["state"] != "booted" and property["class"] == "rootfs":
+                assert property["boot_status"] == "bad"
+
+
 @have_grub
 def test_status_mark_good_dbus(rauc_dbus_service_with_system):
     out, err, exitcode = run("rauc status --output-format=json-pretty")
@@ -73,7 +121,7 @@ def test_status_mark_good_dbus(rauc_dbus_service_with_system):
     out, err, exitcode = run("rauc status mark-good")
 
     assert exitcode == 0
-    assert "marked slot rootfs.0 as good" in out
+    assert "marked slot(s) rootfs.0 as good" in out
 
     out, err, exitcode = run("rauc status --output-format=json-pretty")
     assert exitcode == 0
@@ -97,7 +145,7 @@ def test_status_mark_bad_dbus(rauc_dbus_service_with_system):
     out, err, exitcode = run("rauc status mark-bad")
 
     assert exitcode == 0
-    assert "marked slot rootfs.0 as bad" in out
+    assert "marked slot(s) rootfs.0 as bad" in out
 
     # check post-condition
     out, err, exitcode = run("rauc status --output-format=json-pretty")
@@ -123,7 +171,7 @@ def test_status_mark_active_dbus(rauc_dbus_service_with_system):
     out, err, exitcode = run("rauc status mark-active other")
 
     assert exitcode == 0
-    assert "activated slot rootfs.1" in out
+    assert "rootfs.1 as active" in out
 
     # check post-condition
     out, err, exitcode = run("rauc status --output-format=json-pretty")
